;;; This file is part of guile-with-graphics, a graphic library for Guile

;;; Copyright (C) 2008, 2009  Luca Saiu
;;; Based on turtle, by Luca Saiu

;;; This program is free software: you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation, either version 3 of the License, or
;;; (at your option) any later version.

;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


;; This is the most general version of for, but it's not always
;; comfortable to call. I use this to define three friendlier
;; versions:
(define-macro (for-step variable from to step . body)
  (let ((to-name (gensym))
        (step-name (gensym)))
    `(let ((,variable ,from)
           (,to-name ,to)
           (,step-name ,step))
       (while (<= ,variable ,to-name)
              ,@(append body
                        `((set! ,variable (+ ,variable ,step))))))))

;; Evaluate the body a number of times equal to how-many-times.
;; No iteration variable is visible in the body,
;;
;; Example: (n-times 5 (display "This will be printed five times\n")
(define-macro (n-times how-many-times . body)
  (let ((variable (gensym)))
    `(for ,variable 1 ,how-many-times ,@body)))

;; The classic for loop, with the variable incremented at each iteration.
;;
;; for example (for i 1 5
;;               (display "i is now ")
;;               (display i)
;;               (newline))
;; is equivalent to the C loop:
;; { 
;;   int i;
;;   for(i = 1; i <= 5; i++){
;;     printf("i is now ");
;;     printf("%i", i);
;;     printf("\n");
;;   } // for
;; }
(define-macro (for variable from to . body)
  `(for-step ,variable ,from ,to 1 ,@body))

;; Just like for, but here the variable is *de*cremented at each iteration:
(define-macro (down-for variable from to . body)
  `(for-step ,variable ,from ,to -1 ,@body))

;;; Return any number in the interval [minimum, maximum], with an uniform
;;; probability distribution:
(define (random-from-to minimum maximum)
  (+ (random (- maximum minimum -1))
     minimum))


;; ;;; Try this:
;; (define (rectangles-demo)
;;   (let ((x0 (random-from-to -10 (+ (window-width) 10)))
;;         (y0 (random-from-to -10 (+ (window-height) 10)))
;;         (width (random-from-to 10 (/ (window-width) 2)))
;;         (height (random-from-to 10 (/ (window-height) 2)))
;;         (color (random-color)))
;;     (handle-quit-event-only)
;;     (lock)
;;     (draw-rectangle x0 y0 width height color)
;;     (unlock)
;;     (refresh-rectangle x0 y0 width height)
;;     (rectangles-demo)))

;; ;;; Try this:
;; (define (rectangle-borders-demo)
;;   (let ((x0 (random-from-to -10 (+ (window-width) 10)))
;;         (y0 (random-from-to -10 (+ (window-height) 10)))
;;         (width (random-from-to 10 (/ (window-width) 2)))
;;         (height (random-from-to 10 (/ (window-height) 2)))
;;         (color (random-color)))
;;     (handle-quit-event-only)
;;     (lock)
;;     (draw-rectangle-border x0 y0 width height color)
;;     (unlock)
;;     (refresh-rectangle x0 y0 width height)
;;     (rectangle-borders-demo)))

;; ;;; Try this:
;; (define (lines-demo)
;;   (let ((x1 (random (window-width)))
;;         (y1 (random (window-height)))
;;         (x2 (random (window-width)))
;;         (y2 (random (window-height)))
;;         (color (random-color)))
;;     (handle-quit-event-only)
;;     (lock)
;;     (draw-line x1 y1 x2 y2 color)
;;     (unlock)
;;     (refresh-rectangle x1 y1 (- x2 x1) (- y2 y1))
;;     (lines-demo)))

;; (define (block-grid-demo)
;;   (let* ((block-width 60)
;;          (block-height 20)
;;          (width (window-width))
;;          (height (window-height)))
;;     (handle-quit-event-only)
;;     (for x 0 (/ width block-width)
;;          (for y 0 (/ height block-height)
;;               (lock)
;;               (draw-block (* x block-width) (* y block-height)
;;                           (- block-width 4) (- block-height 4)
;;                           ;;(dark (dark red))
;;                           (random-color)
;;                           3)
;;               (unlock)))
;;     (refresh)
;;     (block-grid-demo)))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; (define block-width 80)
;; (define block-height 25)
;; (define block-color (dark red))
;; (define block-x (/ (window-width) 2))
;; (define block-y (/ (window-height) 2))
;; (define delta-x 0)
;; (define delta-y 0)
;; (define changing-color #f)
;; (define time-at-startup (time))
;; (define tick-length 30)
;; (define speed (* 0.2 tick-length))

;; (define (handle-event event)
;;   (cond ((or (equal? event '(quit))
;;              (equal? event '(key-down #\q))
;;              (equal? event '(key-down escape)))
;;          (exit 0))
;;         ((equal? event '(key-down left))
;;          (set! delta-x (- speed)))
;;         ((equal? event '(key-up left))
;;          (set! delta-x 0))
;;         ((equal? event '(key-down right))
;;          (set! delta-x speed))
;;         ((equal? event '(key-up right))
;;          (set! delta-x 0))
;;         ((equal? event '(key-down down))
;;          (set! delta-y speed))
;;         ((equal? event '(key-up down))
;;          (set! delta-y 0))
;;         ((equal? event '(key-down up))
;;          (set! delta-y (- speed)))
;;         ((equal? event '(key-up up))
;;          (set! delta-y 0))
;;         ((equal? event '(key-up space))
;;          (set! changing-color #f)) 
;;         ((equal? event '(key-down space))
;;          (set! changing-color #t))
;;         (else
;;          (format #t "Unhandled event: ~a\n" event))))

;; (define (on-tick)
;;   (set! block-x (+ block-x delta-x))
;;   (set! block-y (+ block-y delta-y))
;;   (if changing-color
;;       (set! block-color (random-color)))
;;   (draw-block block-x block-y block-width block-height block-color)
;;   (refresh)
;;   ;;(refresh-rectangle (- block-x 1) (- block-y 1) (+ block-width 2) (+ block-height 2))
;;   (draw-rectangle (- block-x 1) (- block-y 1) (+ block-width 2) (+ block-height 2)
;;                   black))


;; (clear blue)
;; (refresh)

;; ;(rectangles-demo)
;; ;(rectangle-borders-demo)
;; ;(blocks-demo)
;; ;(lines-demo)
;; ;(block-grid-demo)

;; (interactive-game tick-length handle-event on-tick)

